function [phi,wy,my,mwy,mx,m2,m3] = est_initial(y,x,w,m)
% Initial consistent estimator for the SARAR model.
% theta = [lambda;rho;beta].
wy = w*y;
phi1 = tsls_sar(w,x,y,wy); % 2SLS estimate.
my = m*y;
mwy = m*wy;
mx = m*x;
m2 = m^2;
m3 = m*m2;
p1 = m + 0.2*m2 + 0.2^2*m3;
p2 = m + 0.6*m2 + 0.6^2*m3;
p1 = p1 - diag(diag(p1));
p2 = p2 - diag(diag(p2));
rho1 = re_rho_initial(y,x,p1,phi1,wy,my,mwy,mx);
rho2 = re_rho_initial(y,x,p2,phi1,wy,my,mwy,mx);
rho_diff = abs([rho1(1)-rho2(1),rho1(1)-rho2(2),rho1(2)-rho2(1),rho1(2)-rho2(2)]);
[~,index] = min(rho_diff);
if index <= 2; rho = rho1(1); else;  rho = rho1(2); end
phi = [phi1(1);rho;phi1(2:end)];

%------------------------------------------------------------------------------
function rho = re_rho_initial(y,x,p,phi1,wy,my,mwy,mx)
% The roots of a quadratic equation of rho with the quadratic matrix P.
u = y-[wy,x]*phi1;
mu = my - [mwy,mx]*phi1;
p2mu = (p+p')*mu;
a = mu'*p2mu/2;
b = -u'*p2mu;
c = u'*p*u;
det = b^2 - 4*a*c;
if det <= 0
    rho = -b*ones(2,1)/(2*a);
else
    rho = [-b-sqrt(det),-b+sqrt(det)]/(2*a);
end

%------------------------------------------------------------------------------
function theta1 = tsls_sar(w,x,y,wy)
% Compute the 2SLS estimate for the SAR model.
wx = w*x;
if  sum(abs(x(:,1)-1))<1e-6 && sum(abs(sum(w,2) -1))<1e-6 % x has an intercept term and w is row-normalized.
    wx(:,1) = [];   
end
f = [x,wx,w*wx]; % IV matrix.
fz = f'*[wy,x];
ff = f'*f;
theta1 = (fz'*(ff\fz))\(fz'*(ff\(f'*y)));