clear; clc; tic;              
% rng(123456);

n_a = [200,400]; % Sample sizes.
lam0_a = [0.2,0.5]; % True values of lambda_0.
rho0_a = [0.2,0.5];  % True values of rho_0. In this Monte Carlo, rho0=lambda0.

iter = 5000; % Number of Monte Carlo repetitions.

sige0 = 1; % True variance.
cl = 0.95; % Confidence level of confidence intervals.
kx = 2; % Number of variables in X.
beta0 = ones(kx,1); % True beta.
kpar = kx+2; % Number of parameters.
models = [length(lam0_a),length(n_a),2]; % Different models. The last dimension is for the two cases without and with heteroskedasticty.
n_est = 3; % 3 estimators.
[est_a,se_a,theta0_a] = deal(zeros([iter,kpar,n_est,models])); % Estimates, standard errors and true parameter values.
fprintf('Monte Carlo repetitions begin now. Current time is %s.\n',datestr(now,'yyyy/mm/dd, HH:MM:SS'));
for i = 1:2
    homo = (i==1); % Homoskedastic or heteroskedastic case.
    for j = 1:length(n_a)
        n = n_a(j);
        w = matrix_hh(n); % Spatial weights matrix.
        m = w;
        if homo
            c_heter = sqrt(sige0);
        else
            heter = sum(w~=0,2); % Heteroskedasticity: Each variance is proportional to the number of nonzero elements in each row.
            c_heter = sqrt(sige0*heter/mean(heter));
        end
        for k = 1:length(lam0_a)
            lam0 = lam0_a(k);
            s0 = speye(n) - lam0*w;
                rho0 = rho0_a(k);
                r0 = speye(n) - rho0*m;
                theta0 = [lam0;rho0;beta0];
                for r = 1:iter %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
                    x = [ones(n,1),randn(n,kx-1)];
                    y = s0\(x*beta0 + r0\((randn(n,1).*c_heter)));
                    if homo
                        [est1,se1] = qmlsarar(y,x,w,m);
                    else
                        [est1,se1] = mm_he(y,x,w,m);
                    end
                    [re,se_re] = re_sarar(y,x,w,m,homo);
                    [re5,se_re5] = re_sarar(y,x,w,m,homo,5);
                    
                    
                    est_a(r,:,:,k,j,i) = [est1,re,re5];
                    se_a(r,:,:,k,j,i) = [se1,se_re,se_re5];
                    theta0_a(r,:,:,k,j,i) = repmat(theta0,1,n_est);
                end
                a = fix(clock);
                fprintf('i = %g (%g), j = %g (%g), k = %g (%g). Current time is %g/%g/%g, %g:%g:%g.\n',...
                    i,2,j,length(n_a),k,length(lam0_a),a(1),a(2),a(3),a(4),a(5),a(6));
        end
    end
end

cp = mean((theta0_a>est_a-se_a*norminv((1+cl)/2)).*(theta0_a<est_a+se_a*norminv((1+cl)/2))); % Coverage probability.
mb = median(est_a-theta0_a); % Median bias.
md = mad(est_a,1); % Median absolute deviation.
idr = quantile(est_a,0.9)-quantile(est_a,0.1); % Interdecile range.

combine4 = @(a,b,c,d) [num2str(a,'% 1.3f'),'(',num2str(b,'% 1.3f'),')',num2str(c,'% 1.3f'),'[',num2str(d,'% 1.3f'),']'];
filename = ['re-',num2str(a(2)),'-',num2str(a(3)),'-',num2str(a(4)),'-',num2str(a(5)),'.xlsx'];
tab1 = reshape(cellfun(combine4,num2cell(mb),num2cell(md),num2cell(idr),num2cell(cp),'UniformOutput',false),kpar,[])';
vn = {'lambda','rho','beta_1','beta_2'}; % Column names.
writetable(cell2table(tab1,'VariableNames',vn),filename,'Sheet',1,'Range','C2');

totaltime = toc;
fprintf('Elapsed time is %g seconds.\n',totaltime);