clear; clc; tstart = tic;
rng(123456789);

n_a = [1e4,1e5,1e6]; % Sample sizes.
lam0_a = [0.2,0.5]; % True values of lambda_0.
rho0_a = [0.2,0.5];  % True values of rho_0.

iter = 5; % Number of Monte Carlo repetitions.

sige0 = 1; % True variance.
kx = 2; % Number of variables in X.
beta0 = ones(kx,1); % True beta.
n_est = 3; % Number of estimators.
models = [length(lam0_a),length(n_a),2]; % Different models. The last dimension is for the two cases without and with heteroskedasticty.
time_a = zeros([n_est,models]); % Average time for compuating each estimate once.
fprintf('Monte Carlo repetitions begin now. Current time is %s.\n',datestr(now,'yyyy/mm/dd, HH:MM:SS'));
for i = 1:2
    homo = (i==1); % Homoskedastic or heteroskedastic case.
    for j = 1:length(n_a)
        n = n_a(j);
        w = matrix_hh(n); % Spatial weights matrix.
        m = w;
        if homo
            c_heter = sqrt(sige0);
        else
            heter = sum(w~=0,2); % Heteroskedasticity: Each variance is proportional to the number of nonzero elements in each row.
            c_heter = sqrt(sige0*heter/mean(heter));
        end
        for k = 1:length(lam0_a)
            lam0 = lam0_a(k);
            s0 = speye(n) - lam0*w;
            % for l = 1:length(rho0_a)
            rho0 = rho0_a(k);
            r0 = speye(n) - rho0*m;
            
            tic;
            for r = 1:iter
                x = [ones(n,1),randn(n,kx-1)];
                y = s0\(x*beta0 + r0\((randn(n,1).*c_heter)));
            end
            time_dgp = toc;
            
            if n <= 1e4 || (n>1e4 && homo) % Do not compute mme for large n.
                tic;
                for r = 1:iter
                    x = [ones(n,1),randn(n,kx-1)];
                    y = s0\(x*beta0 + r0\((randn(n,1).*c_heter)));
                    if homo
                        est1 = qmlsarar(y,x,w,m);
                        fprintf('QML estimate %g.\n',r);
                    else
                        est1 = mm_he(y,x,w,m);
                    end
                end
                time1 = toc - time_dgp;
            else
                time1 = 0;
            end
            a = fix(clock);
            fprintf('Estimate 1 finished. Current time is %g/%g/%g, %g:%g:%g.\n',...
                a(1),a(2),a(3),a(4),a(5),a(6));
            
            if n <= 1e4  % Do not compute re for large n.
                tic;
                for r = 1:iter
                    x = [ones(n,1),randn(n,kx-1)];
                    y = s0\(x*beta0 + r0\((randn(n,1).*c_heter)));
                    re = re_sarar(y,x,w,m,homo);
                end
                time2 = toc - time_dgp;
            else
                time2 = 0;
            end
            a = fix(clock);
            fprintf('Estimate 2 finished. Current time is %g/%g/%g, %g:%g:%g.\n',...
                a(1),a(2),a(3),a(4),a(5),a(6));
            
            tic;
            for r = 1:iter
                x = [ones(n,1),randn(n,kx-1)];
                y = s0\(x*beta0 + r0\((randn(n,1).*c_heter)));
                re = re_sarar(y,x,w,m,homo,5);
            end
            time3 = toc - time_dgp;
            
            time_a(:,k,j,i) = [time1,time2,time3]/iter;
            a = fix(clock);
            fprintf('Estimate 3 finished. Current time is %g/%g/%g, %g:%g:%g. i = %g (%g), j = %g (%g), k = %g (%g).\n',...
                a(1),a(2),a(3),a(4),a(5),a(6),i,2,j,length(n_a),k,length(lam0_a));
            %end
        end
    end
end

time_a = reshape(permute(time_a,[3,1,2,4]),length(n_a),[]);

totaltime = toc(tstart);
fprintf('Elapsed time is %g seconds.\n',totaltime);