clear; clc; tstart = tic;
rng(123456789);

n = 1e4; % Sample sizes.
lam0_a = [0.2,0.5]; % True values of lambda_0.
rho0_a = [0.2,0.5];  % True values of rho_0.

iter = 5; % Number of Monte Carlo repetitions.

sige0 = 1; % True variance.
kx = 2; % Number of variables in X.
beta0 = ones(kx,1); % True beta.
time_a = zeros(1,length(lam0_a)); % Average time for compuating each estimate once.
fprintf('Monte Carlo repetitions begin now. Current time is %s.\n',datestr(now,'yyyy/mm/dd, HH:MM:SS'));
w = matrix_hh(n); % Spatial weights matrix.
m = w;
for k = 1:length(lam0_a)
    lam0 = lam0_a(k);
    s0 = speye(n) - lam0*w;    
    rho0 = rho0_a(k);
    r0 = speye(n) - rho0*m;
    
    tic;
    for r = 1:iter
        x = [ones(n,1),randn(n,kx-1)];
        y = s0\(x*beta0 + r0\((randn(n,1)*sqrt(sige0))));
    end
    time_dgp = toc;
    
    tic;
    for r = 1:iter
        x = [ones(n,1),randn(n,kx-1)];
        y = s0\(x*beta0 + r0\((randn(n,1)*sqrt(sige0))));
        [est1,se1] = qmlsarar(y,x,w,m);
        fprintf('QML estimate %g.\n',r);
    end
    time1 = toc - time_dgp;
    a = fix(clock);
    fprintf('Estimate 1 finished. Current time is %g/%g/%g, %g:%g:%g.\n',...
        a(1),a(2),a(3),a(4),a(5),a(6));
    
    time_a(k) = time1/iter;
    a = fix(clock);
    fprintf('Estimate 3 finished. Current time is %g/%g/%g, %g:%g:%g. k = %g (%g).\n',...
        a(1),a(2),a(3),a(4),a(5),a(6),k,length(lam0_a));
    %end
end

time_a = reshape(permute(time_a,[3,1,2,4]),length(n),[]);

totaltime = toc(tstart);
fprintf('Elapsed time is %g seconds.\n',totaltime);