function [theta,se]=qmlsarar(y,x,w,m,varargin)
% QML estimation of the SARAR model.
if ~isempty(varargin)
    phi_t = varargin{1};
    wy = w*y;
    my = m*y;
    mwy = m*wy;
    mx = m*x;
else
    [phi_t,wy,my,mwy,mx] = est_initial(y,x,w,m);
end
phi_t(1:2) = phi_t(1:2).*(abs(phi_t(1:2))<1) + 0.5*(abs(phi_t(1:2))>=1);

n=length(y);
opts = optimoptions('fmincon','Display','notify','Algorithm','interior-point','SpecifyObjectiveGradient',false);
parm=fmincon(@clsarar,phi_t(1:2),[],[],[],[],[-0.9999,-0.9999],[0.9999,0.9999],[],opts,w,m,x,y,n,wy,my,mwy,mx);
rsy = y - parm(1)*wy - parm(2)*my + parm(1)*parm(2)*mwy;
rx = x - parm(2)*mx;
beta = (rx'*rx)\(rx'*rsy);
theta = [parm;beta]; % The variance parameter is not included.
if nargout>1
    epsilon=rsy-rx*beta;
    [sige,mu3,mu4]=deal(mean(epsilon.^2),mean(epsilon.^3),mean(epsilon.^4));
    mu = my - parm(1)*mwy - mx*beta;
    rwy = wy - parm(2)*mwy;
    r = speye(n)-parm(2)*m;
    s = speye(n)-parm(1)*w;
    mw = m*w;
    rwsr = (w-parm(2)*mw)/(r-parm(1)*w+parm(1)*parm(2)*mw);
    mr = m/r;
    sige2 = sige^2;
    a11 = tr_ab(rwsr,rwsr) + rwy'*rwy/sige;
    a12 = mu'*rwy/sige + epsilon'*mwy/sige;
    a13 = rwy'*rx/sige;
    a14 = epsilon'*rwy/sige2;
    a22 = tr_ab(mr,mr) + mu'*mu/sige;
    a23 = mu'*rx/sige;
    a24 = epsilon'*mu/sige2;
    a33 = rx'*rx/sige;
    a34 = zeros(size(x,2),1);
    a44 = n/(2*sige2);
    Gamma = [a11,a12,a13,a14;a12',a22,a23,a24;a13',a23',a33,a34;a14',a24',a34',a44];
    rwsxb =  r*(w*(s\(x*beta)));
    dev = mu4-3*sige2;
    a11 = tr_ab(rwsr+rwsr',rwsr) + dev*diag(rwsr)'*diag(rwsr)/sige2 + rwsxb'*rwsxb/sige  + rwsxb'*diag(rwsr)*mu3/sige2;
    a12 = tr_ab(rwsr+rwsr',mr) + dev*diag(rwsr)'*diag(mr)/sige2 + rwsxb'*diag(mr)*mu3/sige2;
    a13 = rwsxb'*rx/sige + diag(rwsr)'*rx*mu3/sige2;
    a14 = trace(rwsr)/sige + dev*sum(diag(rwsr))/(2*sige^3) + sum(rwsxb)*mu3/(2*sige^3);
    a22 = tr_ab(mr+mr',mr) + dev*diag(mr)'*diag(mr)/sige2;
    a23 = diag(mr)'*rx*mu3/sige2;
    a24 = trace(mr)/sige + dev*sum(diag(mr))/(2*sige^3);
    a33 = rx'*rx/sige;
    a34 = sum(rx)'*mu3/(2*sige^3);
    a44 =  n/(2*sige2) + dev*n/(4*sige^4);
    Omega = [a11,a12,a13,a14;a12',a22,a23,a24;a13',a23',a33,a34;a14',a24',a34',a44];
    se = sqrt(abs(diag((Gamma\Omega)/Gamma)));
    se(end) = []; % Exclude the standard error for the variance parameter.
end

%------------------------------------------------------------------
function fval = tr_ab(A,B)
% Compute the trace of AB.
fval = sum(A'.*B,'all');

function fval=clsarar(parm,w,m,x,y,n,wy,my,mwy,mx)
% Concentrated objective function.
% w and m are sparse matrices.
rsy = y - parm(1)*wy - parm(2)*my + parm(1)*parm(2)*mwy;
rx = x - parm(2)*mx;
beta = (rx'*rx)\(rx'*rsy);
fval= (n/2)*log(mean((rsy-rx*beta).^2)) - log(abs(det((speye(n) - parm(1)*w)*(speye(n) - parm(2)*m))));